#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/device.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/logging/log.h>
#include "lsm6dsv16x_reg.h"

LOG_MODULE_REGISTER(main, LOG_LEVEL_INF);

// Setup SPI
const struct device *const spi_dev = DEVICE_DT_GET(DT_NODELABEL(spi1));

#define MY_CS DT_NODELABEL(gpio0)
#define GPIO_0_CS  25

// SPI CS control structure
static struct spi_cs_control spi_cs = {
    .gpio = {
        .port = DEVICE_DT_GET(DT_NODELABEL(gpio0)),
        .pin = GPIO_0_CS,
        .dt_flags = GPIO_ACTIVE_LOW,
    },
    .delay = 0,
};

// SPI configuration structure
static struct spi_config spi_cfg = {
  //.frequency = 1000000U,
  .operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB,
  .slave  = 0,
  .cs = &spi_cs,  // Add CS control
};

static int lsm6dsv16x_write(uint8_t reg, uint8_t *data, uint16_t len) {
    uint8_t buffer_tx[1 + len];
    buffer_tx[0] = reg;
    memcpy(&buffer_tx[1], data, len);

    struct spi_buf buf_tx = {
        .buf = buffer_tx,
        .len = sizeof(buffer_tx),
    };

    struct spi_buf_set tx = {
        .buffers = &buf_tx,
        .count = 1,
    };

    return spi_write(spi_dev, &spi_cfg, &tx);
}

static int lsm6dsv16x_read(uint8_t reg, uint8_t *data, uint16_t len) {
    uint8_t buffer_tx[1] = { static_cast<uint8_t>(reg | 0x80) }; // Read operation

    struct spi_buf buf_tx = {
        .buf = buffer_tx,
        .len = sizeof(buffer_tx),
    };

    struct spi_buf_set tx = {
        .buffers = &buf_tx,
        .count = 1,
    };

    struct spi_buf buf_rx = {
        .buf = data,
        .len = len,
    };

    struct spi_buf_set rx = {
        .buffers = &buf_rx,
        .count = 1,
    };

    return spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
}

int main(void) {
    if (!device_is_ready(spi_dev)) {
        printk("SPI device not ready\n");
        return 1;
    }

    if (!device_is_ready(spi_cs.gpio.port)) {
        printk("SPI CS GPIO device not ready\n");
        return 2;
    }

    printk("SPI device ready\n");

    uint8_t who_am_i = 0;
    int ret = lsm6dsv16x_read(LSM6DSV16X_WHO_AM_I, &who_am_i, 1);
    if (ret == 0) {
        printk("LSM6DSV16X WHO_AM_I: 0x%02x\n", who_am_i);
    } else {
        printk("Failed to read WHO_AM_I register\n");
        return 3;
    }

    return 0;
}
